#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "liblihata/dom.h"
#include "liblihata/tree.h"
#include "lht_marshaller.h"

typedef struct rec_s rec_t;

typedef struct rec_s {
	char *s;
	rec_t *next, *children, *parent;
} rec_t;

void print_rec(rec_t *r, int level)
{
	int n;
	do {
		for(n = 0; n < level; n++)
			putchar(' ');
		printf("rec %p: s='%s' parent=%p\n", r, r->s, r->parent);
		if (r->children != NULL)
			print_rec(r->children, level+1);
		r = r->next;
	} while(r != NULL);
}

int main()
{
	lht_doc_t *doc;
	lht_node_t *desc_tree, *data_tree;
	rec_t *root;
	lht_marshaller_ctx_t *ctx;

	char *err;
	doc = lht_dom_load("test_multi.lht", &err);
	if (doc == NULL) {
		fprintf(stderr, "Error loading test_multi.lht: %s\n", err);
		return 1;
	}

	desc_tree = lht_dom_hash_get(doc->root, "types");
	data_tree = lht_dom_hash_get(doc->root, "data");

	ctx = lht_marshaller_ctx_init(desc_tree);
	root = lht_marshaller_ctx_load(ctx, data_tree, "root");

	if (root == NULL)
		printf("Error parsing test_multi.lht\n");
	else
		print_rec(root, 1);

	lht_dom_uninit(doc);
	return 0;
}
