#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include "liblhtview.h"
#include "liblihata/tree.h"

lhtview_t *lhtview_init(int quiet)
{
	lhtview_t *result = malloc(sizeof(lhtview_t));
	if (result != NULL) {
		result->doc = NULL;
		result->cwd = NULL;
	}
	result->quiet = quiet;
	return result;
}

void lhtview_uninit(lhtview_t *ctx)
{
	ctx->cwd = NULL;
	if (ctx->doc != NULL)
		lht_dom_uninit(ctx->doc);
	free(ctx);
}

int lhtview_cmd_load(lhtview_t *ctx, const char *fn, char **errmsg)
{
	if (ctx->doc != NULL) {
		if (errmsg != NULL)
			*errmsg = "a document is already loaded";
		return -1;
	}

	ctx->doc = lht_dom_load(fn, errmsg);
	if (ctx->doc == NULL) {
		if (errmsg != NULL)
			*errmsg = "can not open file for read";
		return -1;
	}

	ctx->cwd = ctx->doc->root;

	if (errmsg != NULL)
		*errmsg = NULL;

	return 0;
}

int lhtview_cmd_cd(lhtview_t *ctx, const char *path, lht_err_t *err)
{
	lht_node_t *result;

	assert(path != NULL);

/* 4th param: 1 == symlink will be followed */
	result = lht_tree_path_(ctx->doc, ctx->cwd, path, 1, 0, err);
	if (result == NULL || result->type == LHT_TEXT) {
		if (err != NULL)
			*err = LHTE_PATH_CHILD_OF_TEXT;
		return -1;
	}

	ctx->cwd = result;
	if (err != NULL)
		*err = LHTE_SUCCESS;
	return 0;
}

int lhtview_cmd_ls(lhtview_t *ctx, FILE *fout, const char *path, int long_format)
{
	lht_dom_iterator_t it;
	lht_node_t *node;
	char *s;
	int needs_brace;

#warning TODO: input parameter "path" not handled at this moment
	if (path != NULL)
		abort();

	if (ctx == NULL || ctx->cwd == NULL)
		return -1;

	for (node = lht_dom_first(&it, ctx->cwd); node != NULL; node = lht_dom_next(&it)) {
		if (long_format)
			fprintf(fout, "%s ", lht_type_name(node->type));

		/* in non-quiet mode brace strings that has leading/trailing whitespace */
		needs_brace = 0;
		if (!ctx->quiet) {
			if (!isspace(*node->name)) {
				needs_brace = 1;
				for(s = node->name; *s != '\0'; s++)
					if (!isspace(*s))
						needs_brace = 0;
					else
						needs_brace = 1;
			}
			else
				needs_brace = 1;

			if (needs_brace)
				fputc('{', fout);
		}

		/* print character-by-character because of escaping:
		    \ and \n is escaped with \
		    } is escaped with \ in needs_brace mode
		*/
		for(s = node->name; *s != '\0'; s++) {
			if ((*s == '\n') || (*s == '\\') || ((needs_brace) && (*s == '}')))
				fputc('\\', fout);
			fputc(*s, fout);
		}

		if (needs_brace)
			fputc('}', fout);
		fputc('\n', fout);
	}

	return 0;
}
